package org.apache.http.impl.cookie;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Locale;
import org.apache.http.FormattedHeader;
import org.apache.http.Header;
import org.apache.http.HeaderElement;
import org.apache.http.NameValuePair;
import org.apache.http.annotation.NotThreadSafe;
import org.apache.http.cookie.Cookie;
import org.apache.http.cookie.CookieAttributeHandler;
import org.apache.http.cookie.CookieOrigin;
import org.apache.http.cookie.MalformedCookieException;
import org.apache.http.impl.cookie.BasicClientCookie;
import org.apache.http.impl.cookie.BasicCommentHandler;
import org.apache.http.impl.cookie.BasicDomainHandler;
import org.apache.http.impl.cookie.BasicExpiresHandler;
import org.apache.http.impl.cookie.BasicMaxAgeHandler;
import org.apache.http.impl.cookie.BasicPathHandler;
import org.apache.http.impl.cookie.BasicSecureHandler;
import org.apache.http.impl.cookie.BrowserCompatSpecFactory;
import org.apache.http.impl.cookie.BrowserCompatVersionAttributeHandler;
import org.apache.http.impl.cookie.CookieSpecBase;
import org.apache.http.impl.cookie.NetscapeDraftHeaderParser;
import org.apache.http.message.BasicHeaderElement;
import org.apache.http.message.BasicHeaderValueFormatter;
import org.apache.http.message.BufferedHeader;
import org.apache.http.message.ParserCursor;
import org.apache.http.util.Args;
import org.apache.http.util.CharArrayBuffer;
import org.apache.http.util.TextUtils;

@NotThreadSafe
public class BrowserCompatSpec extends CookieSpecBase {

   private static final String[] DEFAULT_DATE_PATTERNS = new String[]{"EEE, dd MMM yyyy HH:mm:ss zzz", "EEE, dd-MMM-yy HH:mm:ss zzz", "EEE MMM d HH:mm:ss yyyy", "EEE, dd-MMM-yyyy HH:mm:ss z", "EEE, dd-MMM-yyyy HH-mm-ss z", "EEE, dd MMM yy HH:mm:ss z", "EEE dd-MMM-yyyy HH:mm:ss z", "EEE dd MMM yyyy HH:mm:ss z", "EEE dd-MMM-yyyy HH-mm-ss z", "EEE dd-MMM-yy HH:mm:ss z", "EEE dd MMM yy HH:mm:ss z", "EEE,dd-MMM-yy HH:mm:ss z", "EEE,dd-MMM-yyyy HH:mm:ss z", "EEE, dd-MM-yyyy HH:mm:ss z"};
   private final String[] datepatterns;


   public BrowserCompatSpec(String[] datepatterns, BrowserCompatSpecFactory.SecurityLevel securityLevel) {
      if(datepatterns != null) {
         this.datepatterns = (String[])datepatterns.clone();
      } else {
         this.datepatterns = DEFAULT_DATE_PATTERNS;
      }

      switch(BrowserCompatSpec.SyntheticClass_1.$SwitchMap$org$apache$http$impl$cookie$BrowserCompatSpecFactory$SecurityLevel[securityLevel.ordinal()]) {
      case 1:
         this.registerAttribHandler("path", new BasicPathHandler());
         break;
      case 2:
         this.registerAttribHandler("path", new BasicPathHandler() {
            public void validate(Cookie cookie, CookieOrigin origin) throws MalformedCookieException {
            }
         });
         break;
      default:
         throw new RuntimeException("Unknown security level");
      }

      this.registerAttribHandler("domain", new BasicDomainHandler());
      this.registerAttribHandler("max-age", new BasicMaxAgeHandler());
      this.registerAttribHandler("secure", new BasicSecureHandler());
      this.registerAttribHandler("comment", new BasicCommentHandler());
      this.registerAttribHandler("expires", new BasicExpiresHandler(this.datepatterns));
      this.registerAttribHandler("version", new BrowserCompatVersionAttributeHandler());
   }

   public BrowserCompatSpec(String[] datepatterns) {
      this(datepatterns, BrowserCompatSpecFactory.SecurityLevel.SECURITYLEVEL_DEFAULT);
   }

   public BrowserCompatSpec() {
      this((String[])null, BrowserCompatSpecFactory.SecurityLevel.SECURITYLEVEL_DEFAULT);
   }

   public List parse(Header header, CookieOrigin origin) throws MalformedCookieException {
      Args.notNull(header, "Header");
      Args.notNull(origin, "Cookie origin");
      String headername = header.getName();
      if(!headername.equalsIgnoreCase("Set-Cookie")) {
         throw new MalformedCookieException("Unrecognized cookie header \'" + header.toString() + "\'");
      } else {
         HeaderElement[] helems = header.getElements();
         boolean versioned = false;
         boolean netscape = false;
         HeaderElement[] parser = helems;
         int buffer = helems.length;

         HeaderElement elem;
         for(int cursor = 0; cursor < buffer; ++cursor) {
            elem = parser[cursor];
            if(elem.getParameterByName("version") != null) {
               versioned = true;
            }

            if(elem.getParameterByName("expires") != null) {
               netscape = true;
            }
         }

         if(!netscape && versioned) {
            return this.parse(helems, origin);
         } else {
            NetscapeDraftHeaderParser var19 = NetscapeDraftHeaderParser.DEFAULT;
            CharArrayBuffer var20;
            ParserCursor var21;
            if(header instanceof FormattedHeader) {
               var20 = ((FormattedHeader)header).getBuffer();
               var21 = new ParserCursor(((FormattedHeader)header).getValuePos(), var20.length());
            } else {
               String var22 = header.getValue();
               if(var22 == null) {
                  throw new MalformedCookieException("Header value is null");
               }

               var20 = new CharArrayBuffer(var22.length());
               var20.append(var22);
               var21 = new ParserCursor(0, var20.length());
            }

            elem = var19.parseHeader(var20, var21);
            String name = elem.getName();
            String value = elem.getValue();
            if(name != null && !TextUtils.isBlank(name)) {
               BasicClientCookie cookie = new BasicClientCookie(name, value);
               cookie.setPath(getDefaultPath(origin));
               cookie.setDomain(getDefaultDomain(origin));
               NameValuePair[] attribs = elem.getParameters();

               for(int j = attribs.length - 1; j >= 0; --j) {
                  NameValuePair attrib = attribs[j];
                  String s = attrib.getName().toLowerCase(Locale.ENGLISH);
                  cookie.setAttribute(s, attrib.getValue());
                  CookieAttributeHandler handler = this.findAttribHandler(s);
                  if(handler != null) {
                     handler.parse(cookie, attrib.getValue());
                  }
               }

               if(netscape) {
                  cookie.setVersion(0);
               }

               return Collections.singletonList(cookie);
            } else {
               throw new MalformedCookieException("Cookie name may not be empty");
            }
         }
      }
   }

   private static boolean isQuoteEnclosed(String s) {
      return s != null && s.startsWith("\"") && s.endsWith("\"");
   }

   public List formatCookies(List cookies) {
      Args.notEmpty(cookies, "List of cookies");
      CharArrayBuffer buffer = new CharArrayBuffer(20 * cookies.size());
      buffer.append("Cookie");
      buffer.append(": ");

      for(int headers = 0; headers < cookies.size(); ++headers) {
         Cookie cookie = (Cookie)cookies.get(headers);
         if(headers > 0) {
            buffer.append("; ");
         }

         String cookieName = cookie.getName();
         String cookieValue = cookie.getValue();
         if(cookie.getVersion() > 0 && !isQuoteEnclosed(cookieValue)) {
            BasicHeaderValueFormatter.INSTANCE.formatHeaderElement(buffer, new BasicHeaderElement(cookieName, cookieValue), false);
         } else {
            buffer.append(cookieName);
            buffer.append("=");
            if(cookieValue != null) {
               buffer.append(cookieValue);
            }
         }
      }

      ArrayList var7 = new ArrayList(1);
      var7.add(new BufferedHeader(buffer));
      return var7;
   }

   public int getVersion() {
      return 0;
   }

   public Header getVersionHeader() {
      return null;
   }

   public String toString() {
      return "compatibility";
   }


   // $FF: synthetic class
   static class SyntheticClass_1 {

      // $FF: synthetic field
      static final int[] $SwitchMap$org$apache$http$impl$cookie$BrowserCompatSpecFactory$SecurityLevel = new int[BrowserCompatSpecFactory.SecurityLevel.values().length];


      static {
         try {
            $SwitchMap$org$apache$http$impl$cookie$BrowserCompatSpecFactory$SecurityLevel[BrowserCompatSpecFactory.SecurityLevel.SECURITYLEVEL_DEFAULT.ordinal()] = 1;
         } catch (NoSuchFieldError var2) {
            ;
         }

         try {
            $SwitchMap$org$apache$http$impl$cookie$BrowserCompatSpecFactory$SecurityLevel[BrowserCompatSpecFactory.SecurityLevel.SECURITYLEVEL_IE_MEDIUM.ordinal()] = 2;
         } catch (NoSuchFieldError var1) {
            ;
         }

      }
   }
}
